@@ -1,17 +1,14 @@
-import asyncio
-from asyncio.base_events import BaseEventLoop
-import threading
-from typing import List
-from google.protobuf.descriptor import Descriptor, FieldDescriptor, FileDescriptor, MethodDescriptor, ServiceDescriptor
+from typing import Dict, List
+from google.protobuf.descriptor import FieldDescriptor, FileDescriptor, MethodDescriptor, ServiceDescriptor
 from google.protobuf.json_format import MessageToDict
 from google.protobuf.message import Message
-from grpc import Channel, Server, StatusCode
+from grpc import Channel, StatusCode
 from delphai_utils.logging import logging
 from google.protobuf.descriptor_pb2 import MethodOptions
 from google.api.http_pb2 import HttpRule
 from starlette.applications import Starlette
+from starlette.requests import Request
 from starlette.responses import JSONResponse
-from starlette.routing import Route
 from starlette.exceptions import HTTPException
 from starlette.middleware.base import BaseHTTPMiddleware
 from grpc.experimental.aio import insecure_channel, AioRpcError
@@ -20,13 +17,15 @@ import os
 from google.protobuf import symbol_database
 from hypercorn.config import Config
 from hypercorn.asyncio import serve
-from grpc import StatusCode
 from urllib.parse import urlparse
 from time import perf_counter
+import jwt
+import json
+from base64 import b64encode
 from starlette_prometheus import metrics, PrometheusMiddleware
 from delphai_utils.utils import generate_default_message
 supported_methods = ['get', 'put', 'post', 'delete', 'patch']
-
+user_cache = {}
 status_map = {
   StatusCode.OK: 200,
   StatusCode.CANCELLED: 499,
@@ -60,8 +59,12 @@ class AccessLogMiddleware(BaseHTTPMiddleware):
     return response
 
 
-async def process_grpc_request(descriptor: FileDescriptor, service_name: str, method: str, input: Message,
-                               channel: Channel):
+async def process_grpc_request(descriptor: FileDescriptor,
+                               service_name: str,
+                               method: str,
+                               input: Message,
+                               channel: Channel,
+                               metadata: Dict = {}):
   service = descriptor.services_by_name[service_name]
   method_descriptor = service.methods_by_name[method]
   input_prototype = symbol_database.Default().GetPrototype(method_descriptor.input_type)
@@ -69,7 +72,7 @@ async def process_grpc_request(descriptor: FileDescriptor, service_name: str, me
   method_callable = channel.unary_unary(f'/{service.full_name}/{method}',
                                         request_serializer=input_prototype.SerializeToString,
                                         response_deserializer=output_prototype.FromString)
-  response = await method_callable(input_prototype(**input))
+  response = await method_callable(input_prototype(**input), metadata=metadata.items())
   return response
 
 
@@ -95,13 +98,29 @@ def get_http_handlers(descriptor: FileDescriptor, service_name: str, method: str
     function_name = f'{service.full_name}.{method}'
     return JSONResponse({'function_name': function_name, 'input': input, 'output': output})
 
-  async def request_handler(request):
+  async def request_handler(request: Request):
     body = {}
     if len(await request.body()) > 0:
       body = await request.json()
     input = {**request.path_params, **request.query_params, **body}
     try:
-      raw_output = await process_grpc_request(descriptor, service_name, method, input, channel)
+      metadata = {}
+      if 'Authorization' not in request.headers:
+        logging.warn('no authorization header')
+      else:
+        bearer_token = request.headers['Authorization']
+        if 'x-id-token' in request.headers:
+          logging.info('id_token has been provided')
+          id_token = request.headers['x-id-token']
+          user = jwt.decode(id_token, verify=False)
+          user_json = json.dumps(user).encode('ascii')
+          user_b64 = b64encode(user_json).decode('utf-8')
+          metadata['x-delphai-user'] = user_b64
+        elif bearer_token in user_cache:
+          logging.info('bearer_token in user_cache')
+          metadata['x-delphai-user'] = user_cache[bearer_token]
+
+      raw_output = await process_grpc_request(descriptor, service_name, method, input, channel, metadata=metadata)
       output = MessageToDict(raw_output, preserving_proto_field_name=True)
       return JSONResponse(output)
     except AioRpcError as ex:
